/***********************************************************************
/*
/*
/* REVISED: 04-16-2010   MODIFIED to remove the carriage return and 
/* linefeed for use in the wireless weather stations.
/*
/* The timer and external interrupt will preempt any rx interrupts.  
/* Thus, the rx handler will not affect the anemometer rotation count
/* or time period for samples.
/*
/*
/***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <delays.h>
#include <usart.h>
#include <math.h>
#include <p18l14k22.h>

//#include "main.h"
#include "serial.h"
#include "int_monitor.h"
#include "anemometer.h"


extern struct wind wind;


// declared in anemometer.c
extern volatile unsigned int edge_count;
extern volatile unsigned int working_count;
extern volatile char edge_count_flag;


#pragma interrupt High_isr_handler
void High_isr_handler (void) {
  if(INTCONbits.TMR0IF) Anemometer_timer();
  else if(INTCON3bits.INT2IF) Anemometer_counter();
}

#pragma interrupt Low_isr_handler
void Low_isr_handler (void) {
  if(PIR1bits.RCIF) Usart_rx_handler();
}

#pragma code
void Anemometer_counter (void) {
  if(INTCON3bits.INT2IF) edge_count++;

  INTCON3bits.INT2IF = 0;
  
  return;
}

void Anemometer_timer (void) {
  // Timer 0 is configure to rollover every 3 seconds.

  // Clear timer 0 flag
  INTCONbits.TMR0IF = 0;

  // Disable timer 0
  T0CONbits.TMR0ON = 0;

  // copy the count to a working variable and then clear
  working_count = edge_count;
  edge_count = 0;

  // clear the count flag (flag used in the windsample() function) so that
  // edge count data processing proceeds.
  edge_count_flag = 0;

  // Re-init the timer 0 start count
  TMR0H = 0x48;          // Must write to timer 0 high buffer first
  TMR0L = 0xE4;          // Write to timer 0 low buffer will write to register H:L bytes.

  // While the timer interrupt is serviced, the external interrupt could be pending.  
  // And, the external interrupt isn't valid until the timer is restarted, so clear
  // the external interrupt flag, then if an interrupt is pending, the service routine
  // will test for the flag, find that it isn't set and exit the interrupt.
  INTCON3bits.INT2IF = 0;
  // Re-enable timer 0
  T0CONbits.TMR0ON = 1;

  return;
}

void Usart_rx_handler (void) {
  char temp;

  // Get serial character
  temp = getcUSART();  // USART RX interrupt flag is automatically cleared when RCREG is read.

  // evaluate the RX character
  if (temp == 'H' || temp == 'h') {
     // if 'H'ello, respond with connection ack character 
//     putrsUSART("*\n\r");
     putrsUSART("*");
  } 
  else if (temp == 'R' || temp == 'r') {
     // send the wind speed data   
     putsUSART(wind.windspeed);

  }
  else if (temp == 'U' || temp == 'u') {
     // toggle the units of windspeed   
     if(wind.units) {
       wind.units = mph;
       putrsUSART(" Wind speed miles per hour\n\r");
     }
     else {
       wind.units = mps;
       putrsUSART(" Wind speed meters per second\n\r");
     }

  }
  else { 
//    putrsUSART(" Error1\n\r");
    putrsUSART("Error1");
  }

  return;
}


